/****************************************************************************
 * Copyright (c) 2010-2012, EBM WebSourcing - All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
package com.ebmwebsourcing.easycommons.stream;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

import junit.framework.TestCase;

/**
 * Initial developer(s): Christophe DENEUX
 * 
 * @author cdeneux - Capgemini Sud
 * 
 */
public class ReadInputStreamTest extends TestCase {
    
    private StringBuilder content;

    @Override
    protected void setUp() throws Exception {
        super.setUp();

        this.content = new StringBuilder();
        this.content.append("Hi,\n");
        this.content.append("\n");
        this.content.append("My length is 27 bytes.\n");

    }

    /**
     * Check the processing of
     * <code>{@link ReaderInputStream#available()}</code> after to have read few
     * bytes.
     */
    public void testAvailableAfterLittleRead() {

        try {
            
            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);
            
            final byte[] readBuffer = new byte[3];
            ris.read(readBuffer);
            final int availableBytes = ris.available();
            
            assertTrue(availableBytes > 0);

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of
     * <code>{@link ReaderInputStream#available()}</code> after to have read all
     * bytes.
     */
    public void testAvailableAfterAllRead() {

        try {
            
            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);
            
            final byte[] readBuffer = new byte[1024];
            ris.read(readBuffer);
            final int availableBytes = ris.available();
            
            assertTrue(availableBytes == 0);

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of
     * <code>{@link ReaderInputStream#available()}</code> after to have read all
     * bytes.
     */
    public void testAvailableAfterAllReadPerBlocksLastSmaller() {

        try {

            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);

            this.readPerBlocks(ris, 5);

            final int availableBytes = ris.available();

            assertTrue(availableBytes == 0);

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * after to have read all bytes.
     */
    public void testReadAll() {

        try {
            
            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);
            
            final byte[] readBuffer = new byte[1024];
            final int readBytes = ris.read(readBuffer);
            
            assertEquals(this.content.toString(), new String(readBuffer, 0,
                    readBytes));

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * about reading the reader in two passes.
     */
    public void testReadAllInTwoPasses() {

        try {

            final Reader reader = new StringReader(this.content.toString());
            final ReaderInputStream ris = new ReaderInputStream(reader);
            final StringBuilder result = new StringBuilder();

            byte[] readBuffer = new byte[10];
            int readBytes = ris.read(readBuffer);
            assertEquals(readBuffer.length, readBytes);
            result.append(new String(readBuffer, 0, readBytes));

            readBuffer = new byte[1024];
            readBytes = ris.read(readBuffer);
            result.append(new String(readBuffer, 0, readBytes));

            assertEquals(this.content.toString(), result.toString());

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * about reading the reader byte per byte.
     */
    public void testReadAllByOneByte() {

        final Reader reader = new StringReader(this.content.toString());
        final ReaderInputStream ris = new ReaderInputStream(reader);

        this.readPerBlocks(ris, 1);

    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * about reading the reader bytes blocks per bytes blocks. All blocks have
     * the same size.
     */
    public void testReadAllBySameBytesBlocks() {
        final Reader reader = new StringReader(this.content.toString());
        final ReaderInputStream ris = new ReaderInputStream(reader);

        this.readPerBlocks(ris, 3);
    }

    /**
     * Check the processing of <code>{@link ReaderInputStream#read()}</code>
     * about reading the reader bytes blocks per bytes blocks. The last block is
     * smaller than other ones
     */
    public void testReadAllByBytesBlocksLastSmaller() {
        final Reader reader = new StringReader(this.content.toString());
        final ReaderInputStream ris = new ReaderInputStream(reader);

        this.readPerBlocks(ris, 5);
    }

    private void readPerBlocks(final InputStream ris, final int blockSize) {

        try {

            final StringBuilder result = new StringBuilder();

            byte[] readBuffer = new byte[blockSize];
            int readBytes = 0;
            int charPos = 0; // Use to check that the last itreration returned
            // EOF (-1)
            while (readBytes != -1) {
                readBytes = ris.read(readBuffer);
                assertTrue((charPos >= this.content.length() &&
                                readBytes == -1)
                        || (charPos < this.content.length() &&
                                Math.min(
                                        readBuffer.length,
                                        this.content.length() - result.length()) == readBytes));
                if (readBytes > 0) {
                    result.append(new String(readBuffer, 0, readBytes));
                }
                charPos += blockSize;
            }

            assertEquals(this.content.toString(), result.toString());

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

}
